"""Constants for basespace-autoimport command."""
from typing import Optional

from gencove.constants import Optionals


class BaseSpaceAutoImportOptionals(Optionals):
    """BaseSpaceAutoImportOptionals model"""

    metadata_json: Optional[str]
